import type { Networkish } from '@ethersproject/networks';
import type { BaseProvider, Web3Provider } from '@ethersproject/providers';
import type { Connector, Web3ReactStore } from '@web3-react/types';
import type { ReactNode } from 'react';
import type { Web3ReactHooks, Web3ReactPriorityHooks } from './hooks';
import { getPriorityConnector } from './hooks';
/**
 * @typeParam T - A type argument must only be provided if one or more of the connectors passed to Web3ReactProvider
 * is using `connector.customProvider`, in which case it must match every possible type of this
 * property, over all connectors.
 */
export type Web3ContextType<T extends BaseProvider = Web3Provider> = {
    connector: Connector;
    chainId: ReturnType<Web3ReactPriorityHooks['useSelectedChainId']>;
    accounts: ReturnType<Web3ReactPriorityHooks['useSelectedAccounts']>;
    isActivating: ReturnType<Web3ReactPriorityHooks['useSelectedIsActivating']>;
    account: ReturnType<Web3ReactPriorityHooks['useSelectedAccount']>;
    isActive: ReturnType<Web3ReactPriorityHooks['useSelectedIsActive']>;
    provider: T | undefined;
    ENSNames: ReturnType<Web3ReactPriorityHooks['useSelectedENSNames']>;
    ENSName: ReturnType<Web3ReactPriorityHooks['useSelectedENSName']>;
    hooks: ReturnType<typeof getPriorityConnector>;
};
/**
 * @param children - A React subtree that needs access to the context.
 * @param connectors - Two or more [connector, hooks(, store)] arrays, as returned from initializeConnector.
 * If modified in place without re-rendering the parent component, will result in an error.
 * @param connectorOverride - A connector whose state will be reflected in useWeb3React if set, overriding the
 * priority selection.
 * @param network - An optional argument passed along to `useSelectedProvider`.
 * @param lookupENS - A flag to enable/disable ENS lookups.
 */
export interface Web3ReactProviderProps {
    children: ReactNode;
    connectors: [Connector, Web3ReactHooks][] | [Connector, Web3ReactHooks, Web3ReactStore][];
    connectorOverride?: Connector;
    network?: Networkish;
    lookupENS?: boolean;
}
export declare function Web3ReactProvider({ children, connectors, connectorOverride, network, lookupENS, }: Web3ReactProviderProps): JSX.Element;
export declare function useWeb3React<T extends BaseProvider = Web3Provider>(): Web3ContextType<T>;
